unit RGS_DBInspector;
{ TRGSDBInspector Version 2.01
  Author : Tnis Argus
  Status: Freeware with source
  eMail : Tonis.Argus@mail.ee

  This is an component to view a dataset vertically
  like properties in Delphi-Object Inspector.
  The idea is taken from TObjectInspector created by Jean-Marie Kengne
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons, dsgnintf, Db, DBCtrls, Mask,
  RGS_FieldNotes, RGS_FieldStore;

type
  TRGSDBInspector = class;

{  TOnButtonClick = procedure (Sender: TObject; Field: TField; FieldRect : TRect) of object;
  TOnGetFocus  = procedure (Sender: TObject; aWinControl : TWinControl; Field: TField)  of object;
  TOnLostFocus  = procedure (Sender: TObject; aWinControl : TWinControl; Field: TField)  of object;
}
  TRgsDBInspField = class(TCollectionItem)
  private
    FFieldName: string;
    FFieldKind : TRgsFieldKind;  //Defined in RGS_FieldStore unit
    FReadOnly : Boolean;
    FValueChecked : string;
    FValueUnChecked : string;
    FPickItems : TStrings;
    FHint : string;
    FWinControl : TWinControl;
    FUseKeys : Boolean;
    FKeySeparator : string;
    FOnButtonClick : TOnButtonClick;
    FOnGetFocus : TOnGetFocus;
    FOnLostFocus : TOnLostFocus;
    procedure SetValues(Value : TStrings);
    procedure SetFieldName(const Value: string);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property FieldName: String read FFieldName write SetFieldName;
    property FieldKind : TRgsFieldKind read FFieldKind write FFieldKind;
    property ReadOnly : Boolean read FReadOnly write FReadOnly;
    property ValueChecked : String read FValueChecked write FValueChecked;
    property ValueUnChecked : String read FValueUnChecked write FValueUnChecked;
    property PickItems : TStrings read FPickItems write SetValues;
    property Hint : string read FHint write FHint;
    property WinControl : TWinControl read FWinControl write FWinControl;
    property UseKeys : Boolean read FUseKeys write FUseKeys;
    property KeySeparator : String read FKeySeparator write FKeySeparator;
    property OnButtonClick : TOnButtonClick read FOnButtonClick write FOnButtonClick;
    property OnGetFocus : TOnGetFocus read FOnGetFocus write FOnGetFocus;
    property OnLostFocus : TOnLostFocus read FOnLostFocus write FOnLostFocus;
  end;

  TRgsDBInspFieldClass = class of TRgsDBInspField;

  TRgsDBInspFieldCollection = class(TCollection)
  private
  protected
    FOwner: TPersistent;
    FRGSDBInspector : TRGSDBInspector;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TRgsDBInspField;
    procedure SetItem(Index: Integer; Value: TRgsDBInspField);
  public
    constructor Create(RGSDBInspector : TRGSDBInspector; RgsDBInspFieldClass: TRgsDBInspFieldClass);
//    destructor Destroy override;
    property Fields[Index: Integer]: TRgsDBInspField read GetItem write SetItem; default;
  end;

  TInplaceEdit = class(TCustomMaskEdit)  //DB
  private
    FText : string;
    FNeedsSave : Boolean;
    FOldValue: Variant;
  protected
    procedure CNkeydown(var Message: TWMkeydown); message CN_KEYDOWN;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure setText(F : TField);
  published
  end;

  TRGSDataLink = class(TDataLink)
  private
    FRGSDBInspector: TRGSDBInspector;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
  end;

  TPopupListbox = class;

  TOnEnterRow = procedure (Sender: TObject; Field: TField; RowRect : TRect) of object;
  TOnDrawRow = procedure (Sender: TObject; Field: TField; RowRect : TRect;
                          var LabelColor : TColor; var TextColor : TColor) of object;

  TRgsDBInspMode=(imRecordView,imPrepareFilter,imVisibleFields);
  TRgsDBInspOptions = set of (doLongHint, doShowAlwaysEditor, doCheckAllFields,
                              useColorReqField, useColorCalcField, useColorLabel,
                              useColorText, useColorReadOnly);
  TRgsFieldsAddMode=(faClearExisting, faAddToExisting);

  TRGSDBInspector = class(TCustomListBox)
  private
    FRgsDBInspOptions : TRgsDBInspOptions;
    FFieldsCollection : TRgsDBInspFieldCollection;
    FRgsDBInspMode : TRgsDBInspMode;
    FRGSFieldNotes : TRGSFieldNotes;
    FFieldNotes : Boolean;
    HalfWay: Integer;
    SplitDrag : Boolean;
    EditRect : TRect;
    FAutoCalcSep: Boolean;
    FDataLink : TRGSDataLink;
    FDataSource : TDataSource;
    FActive: Boolean;
    FWinControl : TWinControl;
    FTextMemo : String;
    FTextImage : String;
    FDBEditor: TInplaceEdit;
    FDownArrow : TBitmap;
    FButton : TBitmap;
    FComboButtonBox : TRect;
    FCBDown : Boolean;
    FActiveField : TField;
    FActiveIndex : Integer;
    FActiveItemIndex : Integer;  //Currently selected item in FFieldsCollection
    FCustomWinControl : TWinControl;
    FInvisibleFields : TList;

    FListVisible: Boolean;
    FDataList: TDBLookupListBox;
    FPickList: TPopupListbox;
    FPickValues : TStrings;
    FActiveList: TWinControl;
    FLookupSource: TDatasource;
    FPopupActivated: Boolean;
    FOnEnterRow : TOnEnterRow;
    FFocusedRow : Integer;
    FOnDrawRow : TOnDrawRow;
    FColorReqField : TColor;
    FColorCalcField : TColor;
    FColorLabel : TColor;
    FColorText : TColor;
    FColorReadOnly : TColor;
    procedure SetCollection(const Value: TRgsDBInspFieldCollection);
    Function GetCollectionIndexByFieldName(aFieldName : String) : Integer;
    procedure ListMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DataListKeyDown(Sender: TObject; var Key : word; shift : TShiftState);
    procedure SetAutoCalcSep(Value: Boolean);
    procedure SetInspectorMode(Value: TRgsDBInspMode);
    function GetDataSource : TDataSource;
    procedure SetDataSource(Value : TDataSource);
    function GetReadOnly : Boolean;
    procedure SetReadOnly(Value : Boolean);
    function GetWinControl : TWinControl;
    procedure SetWinControl(Value : TWinControl);
    procedure SetRGSFieldNotes(Value : TRGSFieldNotes);
    procedure ActivateButtonField(ButtonField, CollIdx : Integer; F: TField);
    procedure DBEditOnExit(Sender : TObject);
  protected
    procedure Loaded; override;
    procedure CloseUp(Accept: Boolean);
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure CheckFieldprops(F : TField; Var CheckField : Integer;
                              Var IsChecked : Boolean; Var CollIdx : Integer;
                              Var ButtonField : Integer;  Var CustomField : Integer;
                              Var aHint : String; Var CustomText : String);
    procedure DrawCheckBox(R: TRect; Checked : Boolean);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton;  Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DblClick; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DataSetChanged;
    procedure ActiveChanged;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  public
    HintWin:THintWindow;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CheckFields;
    procedure DefaultHandler(var Message); Override;
    procedure AddInvisibleField(F : TField);
    procedure ClearFieldDefs;
    procedure AddFieldDefItem(aRgsField : TRgsField);
    procedure AddFieldDefs(aRGSFieldStore : TRGSFieldStore; mode : TRgsFieldsAddMode);
  published
    property InspectorMode : TRgsDBInspMode read FRgsDBInspMode write SetInspectorMode;
    property Fields : TRgsDBInspFieldCollection read FFieldsCollection write SetCollection;
    property RGSFieldNotes : TRGSFieldNotes read FRGSFieldNotes write SetRGSFieldNotes;
    property FieldNotes : Boolean read FFieldNotes write FFieldNotes;
    property AutoCalcSep: Boolean read FAutoCalcSep write SetAutoCalcSep;
    property DataSource : TDataSource read GetDataSource write SetDataSource;
    property ReadOnly : Boolean read GetReadOnly write SetReadOnly;
    property WinControl : TWinControl read GetWinControl write SetWinControl;
    property TextMemo : String read FTextMemo write FTextMemo;
    property TextImage : string read FTextImage write FTextImage;
    property Options : TRgsDBInspOptions read FRgsDBInspOptions write FRgsDBInspOptions;
    property ColorReqField : TColor read FColorReqField write FColorReqField;
    property ColorCalcField : TColor read FColorCalcField write FColorCalcField;
    property ColorLabel : TColor read FColorLabel write FColorLabel;
    property ColorText : TColor read FColorText write FColorText;
    property ColorReadOnly : TColor read FColorReadOnly write FColorReadOnly;
    property OnEnterRow : TOnEnterRow read FOnEnterRow write FOnEnterRow;
    property OnDrawRow : TOnDrawRow read FOnDrawRow write FOnDrawRow;
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ItemHeight;
//    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style default lbOwnerDrawFixed;
    property TabOrder;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TPopupListbox = class(TCustomListbox)
  private
    FSearchText: String;
    FSearchTickCount: Longint;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure KeyPress(var Key: Char); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  end;

  TRGSDBInspectorEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

  TDataFieldProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetDataSource: TDataSource;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure GetValueList(List: TStrings);
  end;

procedure Register;

implementation

constructor TRgsDBInspField.Create(Collection: TCollection);
begin
   inherited Create(Collection);
   FPickItems := TStringList.Create;
end;

destructor TRgsDBInspField.Destroy;
begin
   FPickItems.Free;
   inherited;
end;

procedure TRgsDBInspField.Assign(Source: TPersistent);
begin
   if Source is TRgsDBInspField then
      begin
        FieldName := TRgsDBInspField(Source).FieldName;
        FieldKind := TRgsDBInspField(Source).FieldKind;
        ReadOnly := TRgsDBInspField(Source).ReadOnly;
        ValueChecked := TRgsDBInspField(Source).ValueChecked;
        ValueUnChecked := TRgsDBInspField(Source).ValueUnChecked;
        PickItems := TRgsDBInspField(Source).PickItems;
        Hint := TRgsDBInspField(Source).Hint;
        WinControl := TRgsDBInspField(Source).WinControl;
        UseKeys := TRgsDBInspField(Source).UseKeys;
        KeySeparator := TRgsDBInspField(Source).KeySeparator;
        OnButtonClick := TRgsDBInspField(Source).OnButtonClick;
        OnGetFocus := TRgsDBInspField(Source).OnGetFocus;
        OnLostFocus := TRgsDBInspField(Source).OnLostFocus;
      end
   else
     inherited Assign(Source);
end;

function TRgsDBInspField.GetDisplayName: string;
begin
   Result := FFieldName;
   if Result = '' then
     Result := inherited GetDisplayName;
end;

procedure TRgsDBInspField.SetValues(Value : TStrings);
begin
   FPickItems.Assign(Value);
end;

procedure TRgsDBInspField.SetFieldName(const Value: string);
begin
  if FFieldName <> Value then begin
    FFieldName := Value;
  end;
end;


constructor TRgsDBInspFieldCollection.Create(RGSDBInspector : TRGSDBInspector; RgsDBInspFieldClass: TRgsDBInspFieldClass);
begin
  inherited Create(RgsDBInspFieldClass);
  FRGSDBInspector := RGSDBInspector;
end;

function TRgsDBInspFieldCollection.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TRgsDBInspFieldCollection.GetItem(Index: Integer): TRgsDBInspField;
begin
   Result := TRgsDBInspField(inherited GetItem(Index));
end;

procedure TRgsDBInspFieldCollection.SetItem(Index: Integer; Value: TRgsDBInspField);
begin
   inherited SetItem(Index, Value);
end;

procedure TRGSDBInspector.SetCollection(const Value: TRgsDBInspFieldCollection);
begin
   FFieldsCollection.Assign(Value);
end;

//

procedure TPopupListBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or WS_BORDER;
    ExStyle := WS_EX_TOOLWINDOW or WS_EX_TOPMOST;
    WindowClass.Style := CS_SAVEBITS;
  end;
end;

procedure TPopupListbox.CreateWnd;
begin
  inherited CreateWnd;
  Windows.SetParent(Handle, 0);
  CallWindowProc(DefWndProc, Handle, wm_SetFocus, 0, 0);
end;

procedure TPopupListbox.Keypress(var Key: Char);
var
  TickCount: Integer;
begin
  case Key of
    #8, #27: FSearchText := '';
    #32..#255:
      begin
        TickCount := GetTickCount;
        if TickCount - FSearchTickCount > 2000 then
          FSearchText := '';
        FSearchTickCount := TickCount;
        if Length(FSearchText) < 32 then
          FSearchText := FSearchText + Key;
        SendMessage(Handle, LB_SelectString, WORD(-1), Longint(PChar(FSearchText)));
        Key := #0;
      end;
  end;
  inherited Keypress(Key);
end;

procedure TPopupListbox.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  TRGSDBInspector(Owner).CloseUp((X >= 0) and (Y >= 0) and
      (X < Width) and (Y < Height));
end;

constructor TInplaceEdit.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then
    Visible := False;
end;

destructor TInplaceEdit.Destroy;
begin
  inherited Destroy;
end;

procedure TInplaceEdit.CNkeydown(var Message: TWMkeydown);
Var
  Leaving: Boolean;
Begin
  Leaving := False;
  with Message do
  begin
    Case CharCode Of
      VK_RETURN : Leaving := True;
      VK_ESCAPE :
        Begin
          Leaving := True;
          FNeedsSave := False;
        end;
      VK_F2 : If selText <> '' Then Begin
                SelStart := Length(Text);
                SelLength := 0;
              end;
    End;
    if Leaving then
      begin
        Message.Result := 1;
        ValidateEdit;
        If CanFocus Then
          Parent.SetFocus;
        Visible := False;
      end;
  end;
end;

procedure TInplaceEdit.setText(F : TField);
begin
  FText := F.Text;
  FOldValue := F.Value;
  FNeedsSave := True;
//  EditMask := '';
  EditMask := F.EditMask;
  Text := F.Text;
end;

// TRGSDataLink
procedure TRGSDataLink.ActiveChanged;
begin
  if(FRGSDBInspector <> Nil) then
    FRGSDBInspector.ActiveChanged;
end;

procedure TRGSDataLink.DataSetChanged;
begin
  if(FRGSDBInspector <> Nil) then
    FRGSDBInspector.DataSetChanged;
end;

{ TRGSDBInspector }

Function TRGSDBInspector.GetCollectionIndexByFieldName(aFieldName : String) : Integer;
var
  i : Integer;
Begin
  result := -1;
  for i := 0 to FFieldsCollection.Count-1 do
    If AnsiCompareStr(aFieldName, FFieldsCollection[i].FieldName) = 0 Then
      Begin
        result := i;
        Break;
      end;
end;

procedure TRGSDBInspector.CreateParams;
begin
  inherited CreateParams(Params); { call the inherited first }
  with Params do ExStyle := ExStyle or WS_EX_TOOLWINDOW;
end;

constructor TRGSDBInspector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FFieldsCollection := TRgsDBInspFieldCollection.Create(self, TRgsDBInspField);
  FFieldsCollection.FOwner := Self;

  FDataLink := TRGSDataLink.Create;
  FDataLink.FRGSDBInspector := self;
  FDataSource := TDataSource.Create(self);

  FAutoCalcSep := True;
  FRgsDBInspMode := imRecordView;
  FWinControl := nil;

  ItemHeight := 18;
  Color := clBtnFace;
  Style := lbOwnerDrawFixed;
  Width := 160;

  FDownArrow := TBitmap.Create;
  FDownArrow.Width := 7;
  FDownArrow.Height := 4;
  FDownArrow.canvas.Brush.Color := clBtnFace;
  FDownArrow.canvas.Pen.Color := clBtnface;
  FDownArrow.Canvas.Rectangle(0,0,7,4);
  FDownArrow.Canvas.Pen.Width := 0;
  FDownArrow.Canvas.pen.Color := clBlack;
  FDownArrow.Canvas.MoveTo(0, 0);
  FDownArrow.Canvas.LineTo(7, 0);
  FDownArrow.Canvas.MoveTo(1, 1);
  FDownArrow.Canvas.LineTo(6, 1);
  FDownArrow.Canvas.MoveTo(2, 2);
  FDownArrow.Canvas.LineTo(5, 2);
  FDownArrow.Canvas.Pixels[3, 3] := clBlack;

  FButton := TBitmap.Create;
  FButton.Width := 7;
  FButton.Height := 4;
  FButton.canvas.Brush.Color := clBtnFace;
  FButton.canvas.Pen.Color := clBtnface;
  FButton.Canvas.Rectangle(0,0,7,4);
  FButton.Canvas.Pen.Width := 0;
  FButton.Canvas.pen.Color := clBlack;
{  FButton.Canvas.MoveTo(0, 0);
  FButton.Canvas.LineTo(7, 0);
  FButton.Canvas.MoveTo(1, 1);
  FButton.Canvas.LineTo(6, 1);
  FButton.Canvas.MoveTo(2, 2);
  FButton.Canvas.LineTo(5, 2);
}
  FButton.Canvas.Pixels[3, 3] := clBlack;
  FButton.Canvas.Pixels[1, 3] := clBlack;
  FButton.Canvas.Pixels[5, 3] := clBlack;

  FDBEditor := TInplaceEdit.Create(Self);
  FDBEditor.BorderStyle := bsNone;
  FDBEditor.Parent := Self;
  FDBEditor.Visible := False;
  FDBEditor.Left := -200;
  FDBEditor.OnExit := DBEditOnExit;

  FInvisibleFields := TList.Create;
  FLookupSource := TDataSource.Create(Self);
//  FActiveIndex := -1;

  HintWin := THintWindow.Create(self);
  Hintwin.Color := clInfoBk;
  Hintwin.Canvas.Font.Color := clInfoText;
  Options := [doLongHint, doShowAlwaysEditor];
  FFieldNotes := False;
end;

destructor TRGSDBInspector.Destroy;
begin
  FFieldsCollection.Free;
  FInvisibleFields.Free;
  FDownArrow.Free;
  FButton.Free;
  FDBEditor.Free;
  If FPickList <> nil Then
    FPickList.Free;
  If FPickValues <> nil Then
    FPickValues.Free;
  If FLookupSource <> nil Then
    FLookupSource.Free;
  FDataLink.FRGSDBInspector := Nil;
  FDataLink.Free;
  inherited Destroy;
end;

procedure TRGSDBInspector.Loaded;
begin
  inherited;
  ItemIndex := 0;
  FActiveIndex := ItemIndex;
end;


procedure TRGSDBInspector.ListMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) And Not FPopupActivated then
    CloseUp(PtInRect(FActiveList.ClientRect, Point(X, Y)));
  FPopupActivated := False;
end;

procedure TRGSDBInspector.DataListKeyDown(Sender: TObject; var Key : word; shift : TShiftState);
begin
  Case Key of
   VK_RETURN:
     Begin
       CloseUp(True);
       FPopupActivated := False;
     end;
   VK_ESCAPE:
     Begin
       CloseUp(False);
       FPopupActivated := False;
     end;
   end;
end;

procedure TRGSDBInspector.CloseUp(Accept: Boolean);
var
  MasterField: TField;
  ListValue: Variant;
begin
  if FListVisible then
  begin
    if GetCapture <> 0 then
      SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
    if FActiveList = FDataList then
      ListValue := FDataList.KeyValue
    else
      if FPickList.ItemIndex <> -1 then
        ListValue :=  FPickValues[FPicklist.ItemIndex]; //FPickList.Items[FPicklist.ItemIndex];
    SetWindowPos(FActiveList.Handle, 0, 0, 0, 0, 0, SWP_NOZORDER or
      SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE or SWP_HIDEWINDOW);
    FListVisible := False;
    if Assigned(FDataList) then
      FDataList.ListSource := nil;
    FLookupSource.Dataset := nil;
    Invalidate;
    if Accept then
      if FActiveList = FDataList then
        begin
          MasterField := TField(Items.Objects[FActiveIndex]).DataSet.FieldByName(TField(Items.Objects[FActiveIndex]).KeyFields);
          if not FDataLink.ReadOnly And MasterField.CanModify then
            begin
              FDataSource.DataSet.Edit;
              MasterField.Value := ListValue;
            end;
        end
      else
        if (not VarIsNull(ListValue)) and not FDataLink.ReadOnly then
           begin
             MasterField := TField(Items.Objects[FActiveIndex]);
             if not FDataLink.ReadOnly And MasterField.CanModify then
               begin
                 FDataSource.DataSet.Edit;
                 MasterField.Text := ListValue;
               end;
           end;
    if FActiveList = FDataList then
      Begin
        SetFocus;  //Self.
//        FDataList.Free;
//        FDataList := nil;
      end;
    FCBDown := False;
    Repaint;
  end;
end;

procedure TRGSDBInspector.DBEditOnExit(Sender : TObject);
var
  MasterField: TField;
Begin
  If Not (Sender As TInplaceEdit).FNeedsSave Or ((Sender As TInplaceEdit).FText = (Sender As TInplaceEdit).Text) Then
    begin
      (Sender As TInplaceEdit).Visible := False;
      exit;
    end;
  MasterField := TField(Items.Objects[FActiveIndex]);  //ItemIndex
  If MasterField.ReadOnly Then
    exit;
  FDataSource.DataSet.Edit;
  try
    If (Sender As TInplaceEdit).Text = '' Then
      MasterField.Clear  // Value := null
    else
      MasterField.Text := (Sender As TInplaceEdit).Text;
  except
{    If varIsNull((Sender As TInplaceEdit).FOldValue) Then
      MasterField.Clear
    else
      MasterField.Text := (Sender As TInplaceEdit).FOldValue;
}
     MasterField.Clear;
  end;
  (Sender As TInplaceEdit).Visible := False;
end;

procedure TRGSDBInspector.SetAutoCalcSep(Value: Boolean);
begin
  if FAutoCalcSep <> Value then
    FAutoCalcSep := Value;
end;

procedure TRGSDBInspector.SetInspectorMode(Value: TRgsDBInspMode);
begin
  if FRgsDBInspMode <> Value then
    Begin
      FRgsDBInspMode := Value;
//      Repaint;
      CheckFields;
    end;
end;

function TRGSDBInspector.GetDataSource : TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TRGSDBInspector.GetReadOnly : Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TRGSDBInspector.SetReadOnly(Value : Boolean);
begin
  FDataLink.ReadOnly := Value;
  if FDBEditor.Focused then
    SetFocus;
end;

procedure TRGSDBInspector.SetDataSource(Value : TDataSource);
begin
  FDataSource := Value;
  FDataLink.DataSource := Value;
  if Value <> nil then
    Value.FreeNotification(Self);
end;

procedure TRGSDBInspector.ActiveChanged;
var
  i : integer;
begin
  FActive := False;
  FInvisibleFields.Clear;
  If FDataLink.Active Then
    Begin
      for i := 0 to FDataLink.DataSet.FieldCount-1 do
      If not FDataLink.DataSet.Fields[i].Visible And not (doCheckAllFields in Options) Then
        FInvisibleFields.Add(FDataLink.DataSet.Fields[i]);
      FDBEditor.Visible := False;
      FActive := True;
    end;
  CheckFields;
end;

procedure TRGSDBInspector.DataSetChanged;
begin
  CheckFields;
end;

procedure TRGSDBInspector.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  HintWin.ReleaseHandle;
end;

procedure TRGSDBInspector.DefaultHandler(var Message);
var
   msgT:TMsg;
begin
  MsgT.message := TMessage(Message).Msg;
  Inherited;
  if hintwin = nil then
    Exit;
  if (TMessage(Message).Msg = WM_VScroll) or (HintWin.IsHintMsg(MsgT) and
    (MsgT.message <> WM_MouseMove)) then
   HintWin.ReleaseHandle
end;

procedure TRGSDBInspector.AddInvisibleField(F : TField);
begin
  If FInvisibleFields.IndexOf(F) = -1 Then
    FInvisibleFields.Add(F);
end;

function TRGSDBInspector.GetWinControl : TWinControl;
begin
  Result := FWinControl;
end;

procedure TRGSDBInspector.SetWinControl(Value : TWinControl);
begin
  FWinControl := Value;
end;

procedure TRGSDBInspector.SetRGSFieldNotes(Value : TRGSFieldNotes);
begin
  FRGSFieldNotes := Value;
  If Value = nil Then
    FFieldNotes := False;
end;

procedure TRGSDBInspector.ClearFieldDefs;
Begin
  FFieldsCollection.Clear;
end;

procedure TRGSDBInspector.AddFieldDefItem(aRgsField : TRgsField);
var
  aFieldItem : TCollectionItem;
begin
  aFieldItem := FFieldsCollection.Add;
  with aRgsField do begin
    (aFieldItem As TRgsDBInspField).FieldName := DataField;
    (aFieldItem As TRgsDBInspField).FieldKind := FieldKind;
    (aFieldItem As TRgsDBInspField).ReadOnly := ReadOnly;
    (aFieldItem As TRgsDBInspField).ValueChecked := ValueChecked;
    (aFieldItem As TRgsDBInspField).ValueUnChecked := ValueUnChecked;
    (aFieldItem As TRgsDBInspField).PickItems := PickItems;
    (aFieldItem As TRgsDBInspField).Hint := Hint;
    (aFieldItem As TRgsDBInspField).WinControl := WinControl;
    (aFieldItem As TRgsDBInspField).UseKeys := UseKeys;
    (aFieldItem As TRgsDBInspField).KeySeparator := KeySeparator;
    (aFieldItem As TRgsDBInspField).OnButtonClick := OnButtonClick;
    (aFieldItem As TRgsDBInspField).OnGetFocus := OnGetFocus;
    (aFieldItem As TRgsDBInspField).OnLostFocus := OnLostFocus;
  end;
end;

procedure TRGSDBInspector.AddFieldDefs(aRGSFieldStore : TRGSFieldStore; mode : TRgsFieldsAddMode);
var
  i : integer;
begin
  If mode = faClearExisting Then
    ClearFieldDefs;
  for i := 0 to aRGSFieldStore.FieldDefs.Count-1 do
    If aRGSFieldStore.FieldDefs[i].DataSource = FDataSource Then
      AddFieldDefItem(aRGSFieldStore.FieldDefs[i]);
end;

procedure TRGSDBInspector.CheckFields;
var
  X : Integer;
  Field : TField;
  EditorFocused : Boolean;
begin
  if Not (FDataLink.Active) then
    Begin
      Clear;
      exit;
    end;
  If FDataLink.DataSet = nil Then
    Begin
      Clear;
      exit;
    end;
  EditorFocused := FDBEditor.Focused;
  if EditorFocused then
    SetFocus;
  Clear;
    //Invalidate;
  try
    HalfWay := 0;
    if not FAutoCalcSep then
      HalfWay := Width div 2;
    for X := 0 to FDataLink.DataSet.FieldCount-1 do Begin
      Field := FDataLink.DataSet.Fields[X];
      If FInvisibleFields.IndexOf(Field) = -1 Then
        case FRgsDBInspMode of
          imVisibleFields : Begin
                              if FAutoCalcSep then
                                if Canvas.TextWidth(Field.DisplayLabel)+10 > HalfWay then
                                  HalfWay := Canvas.TextWidth(Field.DisplayLabel)+10;
                              Items.AddObject('', Field);
                            end
          else
            if Field.Visible then
              Begin
                if FAutoCalcSep then
                  if Canvas.TextWidth(Field.DisplayLabel)+10 > HalfWay then
                    HalfWay := Canvas.TextWidth(Field.DisplayLabel)+10;
                Items.AddObject('', Field);
              end;
        end;
    end;
  finally
    if EditorFocused then
      Begin
        FDBEditor.Visible := True;
        FDBEditor.SetFocus;
      end;
  end;
end;

procedure TRGSDBInspector.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  EdString, RString: string;
  F: TField;
  S: String;
  SRect: TRect;
  DRect: TRect;
  CheckField : Integer;
  IsChecked : Boolean;
  CollIdx : Integer;
  CustomField, ButtonField : Integer;
  aHint, CustomText : String;
  LabelColor, TextColor : TColor;
  aRGSFieldItem : TRGSFieldItem;
//  aItemIndex : Integer;
begin
  if Assigned(OnDrawItem) then
    OnDrawItem(Self, Index, Rect, State)
  else
  begin
//    HalfWay := Width div 2;
    F := TField(Items.Objects[Index]);
    If F = nil Then
      exit;
    CheckFieldProps(F, CheckField, IsChecked, CollIdx, ButtonField, CustomField, aHint, CustomText);
    HideCaret(Handle);
    If useColorLabel in Options Then
      LabelColor := ColorLabel
    else
      LabelColor := Self.Color;
    If (useColorReadOnly in Options) And F.ReadOnly Then
      LabelColor := ColorReadOnly;
    If (useColorReqField in Options) And F.Required And VarIsNull(F.Value) Then
      LabelColor := ColorReqField;
    If (useColorCalcField in Options) And (F.FieldKind = fkCalculated) Then
      LabelColor := ColorCalcField;
    If FFieldNotes And (FRGSFieldNotes <> nil) Then
      Begin
        aRGSFieldItem := FRGSFieldNotes.GetField(F.DataSet, F.FieldName);
        If aRGSFieldItem <> nil Then
          LabelColor := aRGSFieldItem.Color;
      end;
    If useColorText in Options Then
      TextColor := ColorText
    else
      TextColor := Self.Color;
    If Assigned(OnDrawRow) Then
      OnDrawRow(Self, F, Rect, LabelColor, TextColor);
    S := '';
//    for V := 0 to PEditData.Depth do
      S := S + '  ';
    { draw the item }
    with Canvas do
    begin
      Brush.Color := LabelColor;  {Won't use Windows Selected color.}
      SRect := Rect;
      If odSelected in state then
        SRect.Right := HalfWay;
      FillRect(SRect);
      if (odSelected in State) then
        Pen.Color := clBtnHighLight
      else {for 3D look}
        Pen.Color := clBtnShadow;
      Pen.Mode := pmCopy;
      Pen.Width := 1;
      Pen.Style := psSolid;
      MoveTo(rect.Left, Rect.Bottom-1);
      LineTo(rect.Right, Rect.Bottom-1);
      if (odSelected in State) then
      begin
        Pen.Color := clBtnHighLight;
        MoveTo(HalfWay + 2, Rect.Top+1);
        LineTo(HalfWay + 2, Rect.Bottom);

        Pen.Color := clBtnShadow;
        MoveTo(HalfWay+1, Rect.Top+2);
        LineTo(HalfWay+1, Rect.Bottom);

        MoveTo(Rect.Right, Rect.Top);
        LineTo(Rect.Left, Rect.Top);
        LineTo(Rect.Left, Rect.Bottom);

        Pen.Color := clBlack;
        MoveTo(Rect.Right - 1, Rect.Top + 1);
        LineTo(Rect.Left + 1, Rect.Top + 1);
        LineTo(Rect.Left + 1, Rect.Bottom - 1);
      end else
      begin
        Pen.Color := clBtnShadow;
        MoveTo(HalfWay, Rect.Top);
        LineTo(HalfWay, Rect.Bottom);

        Pen.Color := clBtnHighLight;
        MoveTo(HalfWay + 1, Rect.Top);
        LineTo(HalfWay + 1, Rect.Bottom);
      end;

      SRect.Left := Rect.Left;
      SRect.Right := HalfWay;
      SRect.Top := Rect.Top+1;
      SRect.Bottom := Rect.Bottom-2;  //-1
      Font.Color := clBlack;

      If (odSelected in State) then
      Begin
        Inc(SRect.Left);
        Inc(SRect.Top);
        Inc(SRect.Right);
        If FFocusedRow <> Index Then
          Begin
            FFocusedRow := Index;
            If Assigned(FOnEnterRow) Then
              FOnEnterRow(self, F, Rect);
          end;
      End;
      setBkMode(Canvas.Handle, Transparent);
      TextRect(SRect, SRect.Left, SRect.Top, S + F.DisplayLabel);
      Pen.Color := clBtnFace;
      Font.Color := clNavy;
      If CustomText > '' Then
        RString := CustomText
      else If F.DataType in [ftTypedBinary, ftBlob, ftMemo, ftFmtMemo] Then
        RString := FTextMemo
      else If F.DataType = ftGraphic Then
        RString := FTextImage
      else
        RString := F.DisplayText;  //F.AsString
      Canvas.Font.Color := clNavy; // Text color on the right side
      SRect := ItemRect(ItemIndex);
      if odSelected in state then
       If ButtonField > -1 Then  //(F.FieldKind = fkLookup)
         begin
           FComboButtonBox := SRect;
           SRect.Right := SRect.Right - ABS(SRect.Bottom - SRect.Top);
           Inc(FComboButtonBox.Top, 2);
           FComboButtonBox.Left := SRect.Right + 1;
           DrawButtonFace(Canvas, FComboButtonBox, 3, bsNew, false, false, false);
           DRect.Left := FComboButtonBox.Left +
             ((FComboButtonBox.Right - FComboButtonBox.Left) DIV 2) - (7 DIV 2);
           DRect.Right := DRect.Left + FDownArrow.Width;
           DRect.Top := FComboButtonBox.Top +
             ((FComboButtonBox.Bottom - FComboButtonBox.Top) DIV 2) - (4 DIV 2);
           DRect.Bottom := DRect.Top + FDownArrow.Height;
           If ButtonField < 2 Then
             Canvas.CopyRect(DRect, FDownArrow.Canvas, FDownArrow.Canvas.ClipRect)
           else
             Canvas.CopyRect(DRect, FButton.Canvas, FButton.Canvas.ClipRect);
         End
       else
         If CheckField > -1 Then
           Begin
             EditRect := ItemRect(ItemIndex);
             EditRect.Left := HalfWay + 2;
             DrawCheckBox(EditRect, IsChecked);
           end;
      if (not FDBEditor.Focused) and (odSelected in State) then
        With Canvas do
        Begin
          FDBEditor.Visible := False;
          EditRect := ItemRect(ItemIndex);
          EditRect.Left := HalfWay + 2;

          FDBEditor.Width := EditRect.Right - EditRect.Left;

          Brush.Color := clWhite;
          Inc(EditRect.Top, 2);
          If (ButtonField > -1) Then
            EditRect.Right := EditRect.Right - ABS(EditRect.Bottom - EditRect.Top);
          if (CheckField = -1) And (CustomField = -1) then
            begin
              Fillrect(EditRect);
              setBkColor(Canvas.Handle, GetSysColor(color_HighLight));
              setBkMode(Canvas.Handle, Opaque);
              Font.Color := clHighlightText;
              Canvas.Brush.Color := clHighlight;
              EdString := RString;
              while (TextWidth(EdString) > (EditRect.Right - EditRect.Left)) and
                (EdString <> '')  do
                Delete(EdString, 1, 1);
              TextOut(EditRect.Left + 2, EditRect.Top, edString); //EditRect.Top+1
              DestroyCaret;
              CreateCaret(Handle, 0, 1, (EditRect.Bottom - Editrect.Top) - 2);
              SetCaretPos(EditRect.Left + TextWidth(EdString), EditRect.Top + 1);
              ShowCaret(Handle);
            end;
        end
      else
        begin
          EditRect := ItemRect(Index);  //ItemIndex
          EditRect.Left := HalfWay + 2;
          EditRect.Bottom := EditRect.Bottom - 1;
          If (ButtonField > -1) And (FFocusedRow = Index) Then
            Begin
              EditRect.Right := EditRect.Right - ABS(EditRect.Bottom - EditRect.Top);
              FDBEditor.Width := EditRect.Right - EditRect.Left;
            end;
          Brush.Color := TextColor;
          FillRect(EditRect);
          if CheckField > -1 then  //F.DataType = ftBoolean
            begin
              EditRect := ItemRect(Index);  //ItemIndex
              EditRect.Left := HalfWay + 2;
              DrawCheckBox(EditRect, IsChecked);
            end
          else
            Canvas.TextRect(EditRect, HalfWay + 3, Rect.Top+1, RString);
        End;
      if (odFocused in State) and (odSelected in State) then
        Canvas.DrawFocusRect(Rect);
    end;
  end;
end;

procedure TRGSDBInspector.CheckFieldprops(F : TField; Var CheckField : Integer;
                                          Var IsChecked : Boolean; Var CollIdx : Integer;
                                          Var ButtonField : Integer; Var CustomField : Integer;
                                          Var aHint : String; Var CustomText : String);
var
  i : integer;
  ks, pickVal : string;
Begin
  CheckField := -1;
  ButtonField := -1;  //0 - fkLookup; 1- rgsPickList ; 2- rgsButton
  CustomField := -1;
  CollIdx := -1;
  CustomText := '';
  If FRgsDBInspMode = imVisibleFields Then
    Begin
      CheckField := 1;
      IsChecked := F.Visible;
      exit;
    end;
  If F.DataType = ftBoolean Then
    begin
      CheckField := 0;
      IsChecked := F.AsBoolean;
    end;
  If F.FieldKind = fkLookup Then
    ButtonField := 0;
  CollIdx := GetCollectionIndexByFieldName(F.FieldName);
  If CollIdx > -1 Then
    Begin
      aHint := FFieldsCollection[CollIdx].Hint;
      If (FFieldsCollection[CollIdx].FieldKind = rgsCheckBox) And (CheckField <> 0) Then
        Begin
          CheckField := 1;
          IsChecked := (F.Text = FFieldsCollection[CollIdx].ValueChecked);
        end;
      If FFieldsCollection[CollIdx].FieldKind = rgsPickList Then
        Begin
          ButtonField := 1;
          ks := FFieldsCollection[CollIdx].KeySeparator;
          If FFieldsCollection[CollIdx].UseKeys Then
            Begin
              For i := 0 to FFieldsCollection[CollIdx].PickItems.Count-1 do
                Begin
                  If ks = '' Then
                    pickVal := copy(FFieldsCollection[CollIdx].PickItems[i],1,1)
                  else
                    pickVal := copy(FFieldsCollection[CollIdx].PickItems[i],1,Pos(ks, FFieldsCollection[CollIdx].PickItems[i])-1);
                  if pickVal = F.Text Then
                    Begin
                      If ks = '' Then
                        CustomText := FFieldsCollection[CollIdx].PickItems[i]
                      else
                        CustomText := copy(FFieldsCollection[CollIdx].PickItems[i], Pos(ks, FFieldsCollection[CollIdx].PickItems[i])+1, MaxInt);
                      Break;
                    end;
                end;
            end;
        end;
      If FFieldsCollection[CollIdx].FieldKind = rgsButton Then
        ButtonField := 2;
      If FFieldsCollection[CollIdx].FieldKind = rgsCustomField Then
        CustomField := 0;
    end;
end;

procedure TRGSDBInspector.DrawCheckBox(R: TRect; Checked : Boolean);
var
  DrawState: Integer;
  DrawRect: TRect;
begin
  If Checked Then
    DrawState := DFCS_BUTTONCHECK or DFCS_CHECKED
  else
    DrawState := DFCS_BUTTONCHECK;
  DrawRect.Left := R.Left;
  DrawRect.Right := R.Right;
  DrawRect.Top := R.Top;
  DrawRect.Bottom := R.Bottom-2;
  DrawFrameControl(Canvas.Handle, DrawRect, DFC_BUTTON, DrawState);
end;

procedure TRGSDBInspector.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
  CheckField : Integer;
  IsChecked, ro : Boolean;
  CollIdx : Integer;
  CustomField, ButtonField : Integer;
  aHint, CustomText : string;
begin
  If ItemIndex =-1 Then
    exit;
  CollIdx := ItemIndex;
  If FCustomWinControl <> nil Then
    Begin
      If (FActiveItemIndex > -1) And Assigned(FFieldsCollection[FActiveItemIndex].OnLostFocus) Then
        FFieldsCollection[FActiveItemIndex].OnLostFocus(self, FCustomWinControl, FActiveField);
      FCustomWinControl.Visible := False;
    end;
  FCustomWinControl := nil;
  If FListVisible Then
    CloseUp(PtInRect(FActiveList.ClientRect, Point(X, Y)));
  FActiveIndex := CollIdx;
  FActiveField := TField(Items.Objects[FActiveIndex]);
  If FActiveField = nil Then
    exit;
  CheckFieldprops(FActiveField, CheckField, IsChecked, CollIdx, ButtonField, CustomField, aHint, CustomText);
  FActiveItemIndex := CollIdx;
  If Abs(x - Halfway) <= 3 then
    SplitDrag := True
  else
    Begin
      If FActiveIndex < 0 Then
        exit;
      If (X >= EditRect.left) and (X <= EditRect.Right) and
         (y >= EditRect.Top) and (Y <= EditRect.Bottom) and
         (FActiveField.FieldKind <> fkCalculated) then
      Begin
        If (CollIdx > -1) And (FFieldsCollection[CollIdx].WinControl <> nil) Then
          Begin
//            If FDataLink.ReadOnly And not (doShowAlwaysEditor in Options) Then
//              exit;
            P := Point(EditRect.left, EditRect.Top);  //ClientToScreen(
            FCustomWinControl := FFieldsCollection[CollIdx].WinControl;
            FCustomWinControl.Parent := Self.Parent;
            FCustomWinControl.Top := P.Y;   // EditRect.Top
            FCustomWinControl.Left := P.X;  // EditRect.Left
            FCustomWinControl.Height := (EditRect.Bottom - EditRect.Top);
            FCustomWinControl.Width := (EditRect.Right - EditRect.Left);
//            FCustomWinControl.BoundsRect := EditRect;
            If Assigned(FFieldsCollection[CollIdx].OnGetFocus) Then
              FFieldsCollection[CollIdx].OnGetFocus(self, FCustomWinControl, FActiveField);
            FCustomWinControl.Visible := True;
            FCustomWinControl.BringToFront;
            FCustomWinControl.Show;
            exit;
          end;
        If CheckField > -1 Then  // Boolean
          Begin
            case FRgsDBInspMode of
            imVisibleFields :
              Begin
                IsChecked := Not IsChecked;
                FActiveField.Visible := not FActiveField.Visible;
              end;
            imRecordView :
              Begin
                If Not (FActiveField.ReadOnly Or FDataLink.ReadOnly) Then
                  begin
                    IsChecked := Not IsChecked;
                    FDataSource.DataSet.Edit;
                    If CheckField = 0 Then
                      FActiveField.AsBoolean := not FActiveField.AsBoolean
                    else
                      If FActiveField.Text = FFieldsCollection[CollIdx].ValueChecked Then
                        FActiveField.Text := FFieldsCollection[CollIdx].ValueUnChecked
                      else
                        FActiveField.Text := FFieldsCollection[CollIdx].ValueChecked;
                    DrawCheckBox(EditRect, IsChecked);
                  end;
              end;
            end;
          end
        else
          begin
            ro := FActiveField.ReadOnly Or FDataLink.ReadOnly;
            If (Not ro Or (doShowAlwaysEditor in Options)) And (ButtonField <>1) Then
              Begin
                P.X := x;
                p.y := y;
                Cursor := crDefault;
                FDBEditor.setText(FActiveField);  //TField(Items.Objects[ItemIndex])
                FDBEditor.Left := EditRect.Left;
                FDBEditor.Top := EditRect.Top;
                FDBEditor.Width := EditRect.Right - EditRect.Left;
                FDBEditor.Height := EditRect.Bottom - EditRect.Top;
                FDBEditor.ReadOnly := ro;
                FDBEditor.Visible := True;
                MapWindowPoints(Handle, FDBEditor.Handle, P, 1);
                PostMessage(FDBEditor.Handle, WM_LButtonDown, MK_LButton, P.Y shl 16 + P.X);
              end;
          end;
      End Else
      If (X >= FComboButtonBox.left) and (X <= FComboButtonBox.Right) and
        (y >= FComboButtonBox.Top) and (Y <= FComboButtonBox.Bottom) and (not FCBDown) then
      Begin
        If Not (FActiveField.ReadOnly Or FDataLink.ReadOnly) Or (ButtonField = 2) Then
          ActivateButtonField(ButtonField, CollIdx, FActiveField);
      End;
    end;
end;

procedure TRGSDBInspector.ActivateButtonField(ButtonField, CollIdx : Integer; F: TField);
var
  P: TPoint;
  YY: Integer;
  FieldRect : TRect;
  ks : string;
begin
        If ButtonField = 2 Then
          Begin
            If Assigned(FFieldsCollection[CollIdx].FOnButtonClick) Then
              Begin
                P := ClientToScreen(Point(EditRect.left, EditRect.Top));
                FieldRect.Top := P.Y;
                FieldRect.Left := P.X;
                FieldRect.Right := P.X + FComboButtonBox.Right - EditRect.left;
                FieldRect.Bottom := P.Y + FComboButtonBox.Bottom - EditRect.Top;
                FFieldsCollection[CollIdx].FOnButtonClick(self, F, FieldRect);
              end;
            exit;
          end
        else
        If ButtonField = 1 Then
          Begin
            if FPickList = nil then
              begin
                FPickList := TPopupListbox.Create(Self);
                FPickList.Visible := False;
                FPickList.Parent := Self;
                FPickList.OnMouseUp := ListMouseUp;
                FPickList.IntegralHeight := True;
                FPickList.ItemHeight := 11;
                FPickValues := TStringList.Create;
              end;
            FPickList.Items.Clear;
            FPickList.Items.AddStrings(FFieldsCollection[CollIdx].PickItems);
            FPickValues.Clear;
            FPickValues.AddStrings(FFieldsCollection[CollIdx].PickItems);
            If FFieldsCollection[CollIdx].UseKeys Then
              For YY := 0 to FPickList.Items.Count-1 do
                Begin
                  ks := FFieldsCollection[CollIdx].KeySeparator;
                  If ks = '' Then
                    FPickValues[YY] := copy(FPickValues[YY],1,1)
                  else
                    Begin
                      FPickValues[YY] := copy(FPickValues[YY],1,Pos(ks, FPickValues[YY])-1);
                      FPickList.Items[YY] := copy(FPickList.Items[YY], Pos(ks, FPickList.Items[YY])+1, MaxInt);  //FPickValues
                    end;
                end;
            FPickList.ItemIndex := FPickValues.IndexOf(F.Text);  //FPickList.Items.
            FPickList.Height := FPickList.Items.Count * FPickList.ItemHeight + 4;
            FPickList.Top := EditRect.Bottom;
            FPickList.Left := EditRect.Left;
            FPickList.Width := FComboButtonBox.Right-EditRect.Left;
            FActiveList := FPickList;
          end
        else
          begin
            if FDataList = nil then
              begin
                FDataList := TPopupDataList.Create(Self);  //TDBLookupListBox
                FDataList.Visible := False;
                FDataList.Parent := Self;
                FDataList.OnMouseUp := ListMouseUp;
                FDataList.OnKeyDown := DataListKeyDown;
              end;
    //        FDataList.Color := Color;
    //        FDataList.Font := Font;
            FDataList.RowCount := 7; //Column.DropDownRows;
            FLookupSource.DataSet := F.LookupDataSet;
            FDataList.KeyField := F.LookupKeyFields;
            FDataList.ListField := F.LookupResultField;
            FDataList.ListSource := FLookupSource;
            FDataList.KeyValue := F.DataSet.FieldByName(F.KeyFields).Value;
            FDataList.Top := EditRect.Bottom;
            FDataList.Left := EditRect.Left;
            FDataList.Width := FComboButtonBox.Right-EditRect.Left;
            FActiveList := FDataList;
          end;
        FListVisible := True;
        FDBEditor.Visible := False;
        FCBDown := True;
        P := ClientToScreen(Point(FActiveList.left, FActiveList.Top));  // +FActiveList.Height
        YY := P.Y;// + EditRect.Bottom - EditRect.Top;
        if YY + FActiveList.Height > Screen.Height then
          YY := P.Y - FActiveList.Height;
        SetWindowPos(FActiveList.Handle, HWND_TOP, P.X+2, YY, 0, 0, SWP_NOSIZE or SWP_NOACTIVATE or SWP_SHOWWINDOW);
        FActiveList.Visible := True;
        FActiveList.SetFocus;
        FPopupActivated := True;
end;

procedure TRGSDBInspector.MouseMove(Shift: TShiftState; X, Y: Integer);
Var
  TempR : TRect;
  Ind:integer;
  rct:TRect;
  S, CustomText : String;
  aRGSFieldItem : TRGSFieldItem;
  CheckField : Integer;
  IsChecked : Boolean;
  CollIdx : Integer;
  CustomField, ButtonField : Integer;
  F : TField;
  procedure SetHintRect(S : String);
  begin
          TempR := HintWin.CalcHintRect(Screen.Width,S,nil);
          rct := ItemRect(ind);
          Rct.Right := Rct.Left + TempR.Right - TempR.Left + 9;
          Rct.Top := Rct.Bottom;
          Rct.Bottom := Rct.Top + TempR.Bottom - TempR.Top -1;
          rct.Left := rct.left -1;
          rct.TopLeft := ClientToScreen(rct.TopLeft);
          rct.BottomRight := ClientToScreen(rct.BottomRight);
          If rct.Bottom > Screen.Height Then
            rct.Bottom := Screen.Height - 10;
          If rct.Right > Screen.Width Then
            rct.Right := Screen.Width - 10;
          HintWin.ActivateHint(rct,S);
  end;
begin
  if FActiveIndex = -1 Then
    If Items.count > 0 Then
      Begin
        ItemIndex := 0;
        FActiveIndex := 0;
      end
    else
      exit;
  if Not (doLongHint in Options) then
    HintWin.ReleaseHandle;
  If SplitDrag then
  Begin
    If X > ClientWidth - 20 then
      X := ClientWidth - 20;
    If X < 20 then
      X := 20;
    If Halfway < x then
    Begin
      TempR.Left := Halfway;
      TempR.Right := ClientWidth;
      TempR.Top := 0;
      TempR.Bottom := ClientHeight;
    End Else
    Begin
      TempR.Left := X;
      TempR.Right := ClientWidth;
      TempR.Top := 0;
      TempR.Bottom := ClientHeight;
    End;
    HalfWay := x;
    InvalidateRect(Handle, @tempR, false);
    InvalidateRect(Handle, @EditRect, False);
    PostMessage(Handle, WM_Paint, 0, 0);
    Exit;
  End;
  Ind := ItemAtPos(Point(x,y),true);
  If Ind > -1 Then
    F := TField(Items.Objects[Ind])
  else
    F := nil;
  If Abs(x - HalfWay) <= 3 then
    Cursor := crHSplit
  Else
  If (X >= EditRect.left) and (X <= EditRect.Right) and
     (y >= EditRect.Top) and (Y <= EditRect.Bottom) And (ind > -1) And (F.DataType <> ftBoolean) then
    Cursor := crIBeam
  else
    Cursor := crDefault;
  s := '';
  If FFieldNotes And (FRGSFieldNotes <> nil) And (ind > -1) Then
    Begin
      aRGSFieldItem := FRGSFieldNotes.GetField(F.DataSet, F.FieldName);
      If aRGSFieldItem <> nil Then
        S := aRGSFieldItem.Hint;
    end;
  if (s = '') And (doLongHint in Options) then
    begin
      TempR := ItemRect(ind);
      TempR.Left := Halfway;
      If (ind > -1) And (X >= TempR.left) and (X <= TempR.Right) and
       (y >= TempR.Top) and (Y <= TempR.Bottom) then begin
        S := F.Text;
        if Canvas.textWidth(S) > ((EditRect.Right - EditRect.Left) -3) then
          Begin
          end
        else
          s := '';
      end;
    end;
  If (s='') And (ind > -1) Then
    Begin
      S := '';
      CheckFieldProps(F, CheckField, IsChecked, CollIdx, ButtonField, CustomField, S, CustomText);
    end;
  If not FListVisible And (S > '') Then
    SetHintRect(S)
  else
    HintWin.ReleaseHandle;
  Inherited;
End;

procedure TRGSDBInspector.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  If FListVisible And Not FPopupActivated Then
    CloseUp(PtInRect(FActiveList.ClientRect, Point(X, Y)));
  FPopupActivated := False;
  SplitDrag := False;
end;

procedure TRGSDBInspector.DblClick;
begin
end;

procedure KillMessage(Wnd: HWnd; Msg: Integer);
var
  M: TMsg;
begin
  M.Message := 0;
  if PeekMessage(M, Wnd, Msg, Msg, pm_Remove) and (M.Message = WM_QUIT) then
    PostQuitMessage(M.wparam);
end;

procedure TRGSDBInspector.KeyDown(var Key: Word; Shift: TShiftState);
var
  F: TField;
  CheckField : Integer;
  ro, IsChecked : Boolean;
  CollIdx : Integer;
  CustomField, ButtonField : Integer;
  aHint, CustomText : string;
  aKey: Char;
begin
  If FListVisible Then
    Begin
      aKey := char(Key);
      If Key = VK_RETURN Then
        CloseUp(True)
      else
      If Key = VK_ESCAPE Then
        CloseUp(False)
      else
      If Key = VK_UP Then
        PostMessage(FPickList.Handle, WM_KEYDOWN, VK_UP, 0)
      else
      If Key = VK_DOWN Then
        PostMessage(FPickList.Handle, WM_KEYDOWN, VK_DOWN, 0)
      else
        FPickList.KeyPress(aKey);
      Key:=0;
      KillMessage(Handle, WM_CHAR);
      exit;
    end;
  If ItemIndex = -1 Then
    exit;
  CollIdx := ItemIndex;
  If FCustomWinControl <> nil Then
    Begin
      If (FActiveItemIndex > -1) And Assigned(FFieldsCollection[FActiveItemIndex].OnLostFocus) Then
        FFieldsCollection[FActiveItemIndex].OnLostFocus(self, FCustomWinControl, FActiveField);
      FCustomWinControl.Visible := False;
    end;
  FCustomWinControl := nil;

  F := TField(Items.Objects[ItemIndex]);
  If F = nil Then
    exit;

  FActiveIndex := CollIdx;
  FActiveField := TField(Items.Objects[FActiveIndex]);
  If (FActiveField = nil) Or (FActiveField.FieldKind = fkCalculated) Then
    exit;
  CheckFieldprops(FActiveField, CheckField, IsChecked, CollIdx, ButtonField, CustomField, aHint, CustomText);
  FActiveItemIndex := CollIdx;
 If CheckField > -1 Then  // Boolean
    Begin
      If (Key = VK_SPACE) And Not (F.ReadOnly Or FDataLink.ReadOnly) Then
        begin
          IsChecked := Not IsChecked;
          FDataSource.DataSet.Edit;
          If CheckField = 0 Then
            F.AsBoolean := not F.AsBoolean
          else
            If F.Text = FFieldsCollection[CollIdx].ValueChecked Then
              F.Text := FFieldsCollection[CollIdx].ValueUnChecked
            else
              F.Text := FFieldsCollection[CollIdx].ValueChecked;
          DrawCheckBox(EditRect, IsChecked);
        end;
    end
  else
  If ButtonField > -1 Then  //F.FieldKind = fkLookup
    Begin
      If ((Key = VK_DOWN) And (ssAlt in Shift)) or (Key = VK_SPACE) then
      begin
        ActivateButtonField(ButtonField, CollIdx, F);
        Key := 0;
        Exit;
      end;
      if not (Key in [VK_UP, VK_DOWN, VK_PRIOR, VK_NEXT]) then
      begin
      End;
    end
  else
    Begin
      If (Key = VK_RETURN) or (Key = VK_SPACE) then
      begin
        Click;
        Key := 0;
        Exit;
      end;
      if not (Key in [VK_UP, VK_DOWN, VK_PRIOR, VK_NEXT]) then
      begin
        ro := FActiveField.ReadOnly Or FDataLink.ReadOnly;
        If Not ro Or (doShowAlwaysEditor in Options) Then
          Begin
            FDBEditor.setText(FActiveField);
            FDBEditor.Left := EditRect.Left;
            FDBEditor.Top := EditRect.Top;
            FDBEditor.Width := EditRect.Right - EditRect.Left;
            FDBEditor.Height := EditRect.Bottom - EditRect.Top;
            FDBEditor.ReadOnly := ro;
            FDBEditor.Visible := True;
            PostMessage(FDBEditor.Handle, WM_KeyDown, key, 0);
            FDBEditor.SetFocus;
          end;
      End
      else
        inherited;
    end;
  CollIdx := ItemIndex;
end;

function TRGSDBInspectorEditor.GetVerbCount: integer;
begin
   Result := 1;
end;

function TRGSDBInspectorEditor.GetVerb(Index: integer): string;
begin
   Result := 'Fields editor...';
end;

procedure TRGSDBInspectorEditor.ExecuteVerb(Index: integer);
begin
  Designer.Modified;
end;

function TDataFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];  //, paMultiSelect
end;

procedure TDataFieldProperty.GetValues(Proc: TGetStrProc);
var
  i: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do
      Proc(Values[i]);
  finally
    Values.Free;
  end;
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  i: Integer;
  DataSource: TDataSource;
begin
  DataSource := GetDataSource;
  if (DataSource <> nil) And (DataSource.DataSet <> nil) then
    for i := 0 to DataSource.DataSet.FieldCount - 1 do
        List.Add(DataSource.DataSet.Fields[i].FieldName);
end;

function TDataFieldProperty.GetDataSource: TDataSource;
begin
  with GetComponent(0) as TRgsDBInspField do
    Result := (Collection As TRgsDBInspFieldCollection).FRGSDBInspector.DataSource;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TRGSDBInspector]);
  RegisterComponentEditor(TRGSDBInspector, TRGSDBInspectorEditor);
  RegisterPropertyEditor(TypeInfo(string), TRgsDBInspField, 'FieldName', TDataFieldProperty);
end;

Begin

end.
