unit RGS_FieldNotes;
{ TRGSFieldStore Version 1.0
  Author : Tnis Argus
  Status: Freeware with source
  eMail : Tonis.Argus@mail.ee

  This is an component to store additional information
  related to dataset fields, for example validation results
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB,
  dsgnintf;

type
  TRGSFieldItem = class(TCollectionItem)
  private
    FDataSet: TDataSet;
    FFieldName: String;
    FHint: String;
    FColor : TColor;
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property DataSet: TDataSet read FDataSet write FDataSet;
    property FieldName: string read FFieldName write FFieldName;
    property Hint: String read FHint write FHint;
    property Color : TColor read FColor write FColor;
  end;

  TFieldNotes = class(TCollection)
  protected
    FOwner: TPersistent;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TRGSFieldItem;
    procedure SetItem(Index: Integer; Value: TRGSFieldItem);
  public
    property Items[Index: Integer]: TRGSFieldItem read GetItem write SetItem; default;
  end;

  TRGSFieldNotes = class(TComponent)
  private
    FFieldNotes: TFieldNotes;
  protected
    procedure SetCollection(const Value: TFieldNotes);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure Add(aDataSet : TDataSet; aFieldName, aHint : string; aColor : TColor);
    Procedure AddField(aDataSet : TDataSet; aField : TField; aHint : string; aColor : TColor);
    Procedure ClearAll;
    Procedure ClearDataSet(DataSet : TDataSet);
    Procedure ClearField(DataSet : TDataSet; FieldName : string);
    Function GetField(aDataSet : TDataSet; aFieldName : string) : TRGSFieldItem;
  published
    property FieldNotes: TFieldNotes read FFieldNotes write SetCollection;
  end;

  TRGSFieldNotesEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

  TFieldNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetDataSet: TDataSet;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure GetValueList(List: TStrings);
  end;

procedure Register;

implementation

constructor TRGSFieldItem.Create(Collection: TCollection);
begin
   inherited Create(Collection);
end;

destructor TRGSFieldItem.Destroy;
begin
   inherited;
end;

procedure TRGSFieldItem.Assign(Source: TPersistent);
begin
   if Source is TRGSFieldItem then
      begin
        FieldName := TRGSFieldItem(Source).FieldName;
        Hint := TRGSFieldItem(Source).Hint;
        Color := TRGSFieldItem(Source).Color;
      end
   else
     inherited Assign(Source);
end;

function TRGSFieldItem.GetDisplayName: string;
begin
   Result := FieldName;
   if Result = '' then
     Result := inherited GetDisplayName;
end;

function TFieldNotes.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TFieldNotes.GetItem(Index: Integer): TRGSFieldItem;
begin
   Result := TRGSFieldItem(inherited GetItem(Index));
end;

procedure TFieldNotes.SetItem(Index: Integer; Value: TRGSFieldItem);
begin
   inherited SetItem(Index, Value);
end;

constructor TRGSFieldNotes.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FFieldNotes := TFieldNotes.Create(TRGSFieldItem);
   FFieldNotes.FOwner := Self;
end;

destructor TRGSFieldNotes.Destroy;
begin
   FFieldNotes.Free;
   inherited Destroy;
end;

procedure TRGSFieldNotes.SetCollection(const Value: TFieldNotes);
begin
  FFieldNotes.Assign(Value);
end;

procedure TRGSFieldNotes.Loaded;
begin
   inherited;
end;

Procedure TRGSFieldNotes.Add(aDataSet : TDataSet; aFieldName, aHint : string; aColor : TColor);
var
  i : integer;
  idx : integer;
  aFieldItem : TCollectionItem;
begin
  idx := -1;
  for i := 0 to FFieldNotes.Count-1 do
    If (FFieldNotes[i].DataSet = aDataSet) And (FFieldNotes[i].FieldName = aFieldName) Then
      idx := i;
  If idx = -1 Then
    aFieldItem := FFieldNotes.Add
  else
    aFieldItem := FFieldNotes[idx];
  with aFieldItem As TRGSFieldItem do
    begin
      DataSet := aDataSet;
      FieldName := aFieldName;
      Hint := aHint;
      Color := aColor;
    end;
end;

Procedure TRGSFieldNotes.AddField(aDataSet : TDataSet; aField : TField; aHint : string; aColor : TColor);
var
  i : integer;
  idx : integer;
  aFieldItem : TCollectionItem;
begin
  idx := -1;
  for i := 0 to FFieldNotes.Count-1 do
    If (FFieldNotes[i].DataSet = aDataSet) And (FFieldNotes[i].FieldName = aField.FieldName) Then
      idx := i;
  If idx = -1 Then
    aFieldItem := FFieldNotes.Add
  else
    aFieldItem := FFieldNotes[idx];
  with aFieldItem As TRGSFieldItem do
    begin
      DataSet := aDataSet;
      FieldName := aField.FieldName;
      Hint := aHint;
      Color := aColor;
    end;
end;

Procedure TRGSFieldNotes.ClearAll;
begin
  FFieldNotes.Clear;
end;

Procedure TRGSFieldNotes.ClearDataSet(DataSet : TDataSet);
var
  i : integer;
begin
  i := 0;
  while i < FFieldNotes.Count do
    If FFieldNotes[i].DataSet = DataSet Then
      FFieldNotes[i].Destroy
    else
      inc(i);
end;

Procedure TRGSFieldNotes.ClearField(DataSet : TDataSet; FieldName : string);
var
  i : integer;
begin
  for i := 0 to FFieldNotes.Count-1 do
    If (FFieldNotes[i].DataSet = DataSet) And (FFieldNotes[i].FieldName = FieldName) Then
      FFieldNotes[i].Destroy;
end;

Function TRGSFieldNotes.GetField(aDataSet : TDataSet; aFieldName : string) : TRGSFieldItem;
var
  i : integer;
  idx : integer;
begin
  idx := -1;
  for i := 0 to FFieldNotes.Count-1 do
    If (FFieldNotes[i].DataSet = aDataSet) And (FFieldNotes[i].FieldName = aFieldName) Then
      idx := i;
  If idx = -1 Then
    result := nil
  else
    result := FFieldNotes[idx];
end;

function TRGSFieldNotesEditor.GetVerbCount: integer;
begin
   Result := 1;
end;

function TRGSFieldNotesEditor.GetVerb(Index: integer): string;
begin
   Result := 'Field notes...';
end;

procedure TRGSFieldNotesEditor.ExecuteVerb(Index: integer);
begin
  Designer.Modified;
end;

function TFieldNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TFieldNameProperty.GetValues(Proc: TGetStrProc);
var
  i: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do
      Proc(Values[i]);
  finally
    Values.Free;
  end;
end;

procedure TFieldNameProperty.GetValueList(List: TStrings);
var
  i: Integer;
  DataSet: TDataSet;
begin
  DataSet := GetDataSet;
  if (DataSet <> nil) then
    for i := 0 to DataSet.FieldCount - 1 do
      List.Add(DataSet.Fields[i].FieldName);
end;

function TFieldNameProperty.GetDataSet: TDataSet;
begin
  with GetComponent(0) as TRGSFieldItem do
    Result := DataSet;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TRGSFieldNotes]);
  RegisterComponentEditor(TRGSFieldNotes, TRGSFieldNotesEditor);
  RegisterPropertyEditor(TypeInfo(string), TRGSFieldItem, 'FieldName', TFieldNameProperty);
end;

end.