unit RGS_FieldStore;
{ TRGSFieldStore Version 1.0
  Author : Tnis Argus
  Status: Freeware with source
  eMail : Tonis.Argus@mail.ee

  This is an component to store field information
  for future use in TRGSDBInspector
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB, dsgnintf;
type
  TRgsFieldKind=(rgsPickList,rgsButton,rgsCheckBox,rgsCustomField);
  TRGSFieldStore = class;

  TOnButtonClick = procedure (Sender: TObject; Field: TField; FieldRect : TRect) of object;
  TOnGetFocus  = procedure (Sender: TObject; aWinControl : TWinControl; Field: TField)  of object;
  TOnLostFocus  = procedure (Sender: TObject; aWinControl : TWinControl; Field: TField)  of object;

  TRgsField = class(TCollectionItem)
  private
    FDataSource : TDataSource;
    FFieldName: string;
    FFieldKind : TRgsFieldKind;
    FReadOnly : Boolean;
    FValueChecked : string;
    FValueUnChecked : string;
    FPickItems : TStrings;
    FHint : string;
    FWinControl : TWinControl;
    FUseKeys : Boolean;
    FKeySeparator : string;
    FOnButtonClick : TOnButtonClick;
    FOnGetFocus : TOnGetFocus;
    FOnLostFocus : TOnLostFocus;
    procedure SetValues(Value : TStrings);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    function GetDataField: string;
    procedure SetDataField(const Value: string);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly : Boolean read FReadOnly write FReadOnly;
    property FieldKind : TRgsFieldKind read FFieldKind write FFieldKind;
    property ValueChecked : String read FValueChecked write FValueChecked;
    property ValueUnChecked : String read FValueUnChecked write FValueUnChecked;
    property PickItems : TStrings read FPickItems write SetValues;
    property Hint : string read FHint write FHint;
    property WinControl : TWinControl read FWinControl write FWinControl;
    property UseKeys : Boolean read FUseKeys write FUseKeys;
    property KeySeparator : String read FKeySeparator write FKeySeparator;
    property OnButtonClick : TOnButtonClick read FOnButtonClick write FOnButtonClick;
    property OnGetFocus : TOnGetFocus read FOnGetFocus write FOnGetFocus;
    property OnLostFocus : TOnLostFocus read FOnLostFocus write FOnLostFocus;
  end;

  TRgsFieldClass = class of TRgsField;

  TRgsFieldCollection = class(TCollection)
  protected
    FOwner: TPersistent;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TRgsField;
    procedure SetItem(Index: Integer; Value: TRgsField);
  public
    property Fields[Index: Integer]: TRgsField read GetItem write SetItem; default;
  end;

  TRGSFieldStore = class(TComponent)
  private
    FFieldsCollection: TRgsFieldCollection;
  protected
    procedure SetCollection(const Value: TRgsFieldCollection);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddPickListField(aDataSource : TDataSource; aDataField : string;
                               aPickList : TStrings; ReadOnly : Boolean;
                               useKeys : Boolean; KeySeparator : string);
    procedure AddBooleanField(aDataSource : TDataSource; aDataField : string;
                              ValueChecked, ValueUnChecked : string; ReadOnly : Boolean);
  published
    property FieldDefs: TRgsFieldCollection read FFieldsCollection write SetCollection;
  end;

  TRGSFieldStoreEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

  TDataFieldProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetDataSource: TDataSource;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure GetValueList(List: TStrings);
  end;

procedure Register;

implementation

constructor TRgsField.Create(Collection: TCollection);
begin
   inherited Create(Collection);
   FPickItems := TStringList.Create;
end;

destructor TRgsField.Destroy;
begin
   FPickItems.Free;
   inherited;
end;

procedure TRgsField.Assign(Source: TPersistent);
begin
   if Source is TRgsField then
      begin
        DataSource := TRgsField(Source).DataSource;
        DataField := TRgsField(Source).DataField;
        ReadOnly := TRgsField(Source).ReadOnly;
        FieldKind := TRgsField(Source).FieldKind;
        ValueChecked := TRgsField(Source).ValueChecked;
        ValueUnChecked := TRgsField(Source).ValueUnChecked;
        PickItems := TRgsField(Source).PickItems;
        Hint := TRgsField(Source).Hint;
        WinControl := TRgsField(Source).WinControl;
        UseKeys := TRgsField(Source).UseKeys;
        KeySeparator := TRgsField(Source).KeySeparator;
        OnButtonClick := TRgsField(Source).OnButtonClick;
        OnGetFocus := TRgsField(Source).OnGetFocus;
        OnLostFocus := TRgsField(Source).OnLostFocus;
      end
   else
     inherited Assign(Source);
end;

function TRgsField.GetDisplayName: string;
begin
   Result := FFieldName;
   if Result = '' then
     Result := inherited GetDisplayName;
end;

procedure TRgsField.SetValues(Value : TStrings);
begin
   FPickItems.Assign(Value);
end;

function TRgsField.GetDataSource: TDataSource;
begin
  Result := FDataSource;
end;

procedure TRgsField.SetDataSource(Value: TDataSource);
begin
  FDataSource := Value;
end;

function TRgsField.GetDataField: string;
begin
  Result := FFieldName;
end;

procedure TRgsField.SetDataField(const Value: string);
begin
  FFieldName := Value;
end;

function TRgsFieldCollection.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TRgsFieldCollection.GetItem(Index: Integer): TRgsField;
begin
   Result := TRgsField(inherited GetItem(Index));
end;

procedure TRgsFieldCollection.SetItem(Index: Integer; Value: TRgsField);
begin
   inherited SetItem(Index, Value);
end;


constructor TRGSFieldStore.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FFieldsCollection := TRgsFieldCollection.Create(TRgsField);
   FFieldsCollection.FOwner := Self;
end;

destructor TRGSFieldStore.Destroy;
begin
   FFieldsCollection.Free;
   inherited Destroy;
end;

procedure TRGSFieldStore.SetCollection(const Value: TRgsFieldCollection);
begin
   FFieldsCollection.Assign(Value);
end;

procedure TRGSFieldStore.AddPickListField(aDataSource : TDataSource; aDataField : string;
                                          aPickList : TStrings; ReadOnly : Boolean;
                                          useKeys : Boolean; KeySeparator : string);
var
  aFieldItem : TCollectionItem;
begin
  aFieldItem := FFieldsCollection.Add;
  (aFieldItem As TRgsField).DataSource := aDataSource;
  (aFieldItem As TRgsField).DataField := aDataField;
  (aFieldItem As TRgsField).FieldKind := rgsPickList;
  (aFieldItem As TRgsField).PickItems.AddStrings(aPickList);
  (aFieldItem As TRgsField).UseKeys := UseKeys;
  (aFieldItem As TRgsField).KeySeparator := KeySeparator;
  (aFieldItem As TRgsField).ReadOnly := ReadOnly;
end;

procedure TRGSFieldStore.AddBooleanField(aDataSource : TDataSource; aDataField : string;
                                         ValueChecked, ValueUnChecked : string; ReadOnly : Boolean);
var
  aFieldItem : TCollectionItem;
begin
  aFieldItem := FFieldsCollection.Add;
  (aFieldItem As TRgsField).DataSource := aDataSource;
  (aFieldItem As TRgsField).DataField := aDataField;
  (aFieldItem As TRgsField).FieldKind := rgsCheckBox;
  (aFieldItem As TRgsField).ValueChecked := ValueChecked;
  (aFieldItem As TRgsField).ValueUnChecked := ValueUnChecked;
  (aFieldItem As TRgsField).ReadOnly := ReadOnly;
end;

function TRGSFieldStoreEditor.GetVerbCount: integer;
begin
   Result := 1;
end;

function TRGSFieldStoreEditor.GetVerb(Index: integer): string;
begin
   Result := 'Fields editor...';
end;

procedure TRGSFieldStoreEditor.ExecuteVerb(Index: integer);
begin
  Designer.Modified;
end;

function TDataFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paMultiSelect];
end;

procedure TDataFieldProperty.GetValues(Proc: TGetStrProc);
var
  i: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do
      Proc(Values[i]);
  finally
    Values.Free;
  end;
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  i: Integer;
  DataSource: TDataSource;
begin
  DataSource := GetDataSource;
  if (DataSource <> nil) And (DataSource.DataSet <> nil) then
    for i := 0 to DataSource.DataSet.FieldCount - 1 do
        List.Add(DataSource.DataSet.Fields[i].FieldName);
end;

function TDataFieldProperty.GetDataSource: TDataSource;
begin
  with GetComponent(0) as TRgsField do
    Result := DataSource;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TRGSFieldStore]);
  RegisterComponentEditor(TRGSFieldStore, TRGSFieldStoreEditor);
  RegisterPropertyEditor(TypeInfo(string), TRgsField, 'DataField', TDataFieldProperty);
end;

end.
